// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifndef windows
	#include <gsl/gsl_multimin.h>
#endif

#include "../../sim.h"
#include "../../dump.h"
#include "../../ntricks.h"
#include "../../inp.h"
#include "../../buffer.h"
#include "imps.h"
#include "../../dynamic_store.h"

static int unused __attribute__((unused));

struct imps imps_config;

struct istruct out_i;
struct istruct fit_data;

void make_sin(struct istruct *in,double mag,double fx,double delta)
{
int i;
for (i=0;i<in->len;i++)
{
	in->data[i]=fabs(mag)*sin(2*PI*fx*(in->x[i]+delta));
}

}

char *fit_file_prefix;

double last_delta=0.0;
double last_mag=0.0;

#ifndef windows
double imps_f (const gsl_vector *v, void *params)
{
struct buffer buf;
buffer_init(&buf);
buf.norm_x_axis=FALSE;
char name[200];
double val;
double e0;
int fitcount=0;
int fitvars=1;
double mag = gsl_vector_get(v, 0);
double delta = gsl_vector_get(v, 1);
double *p = (double *)params;
double fit_fx=p[0];
struct istruct test_i;
struct istruct norm_i;
inter_copy(&test_i,&fit_data,TRUE);
double avg=inter_avg(&test_i);
inter_sub_double(&test_i,avg);

buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current origonal- Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$A$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,fit_data.x, fit_data.data, fit_data.len);
sprintf(name,"imps_fit_%s_orig.dat",fit_file_prefix);
buffer_dump("./",name,&buf);
buffer_free(&buf);


make_sin(&test_i,mag,fit_fx,delta);
//inter_save(&test_i,"./rod_test.dat");
//inter_save(&fit_data,"./rod_test2.dat");

buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current guess - Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$A$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,test_i.x, test_i.data, test_i.len);
sprintf(name,"imps_fit_%s_guess.dat",fit_file_prefix);
buffer_dump("./",name,&buf);
buffer_free(&buf);

inter_sub(&test_i,&fit_data);
buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current fit error - Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$A$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,test_i.x, test_i.data, test_i.len);
sprintf(name,"./imps_fit_%s_delta.dat",fit_file_prefix);
buffer_dump("./",name,&buf);
buffer_free(&buf);

inter_mod(&test_i);
e0=inter_intergrate(&test_i);
inter_free(&test_i);
printf("e0=%le %le %le %le\n",e0,mag, delta,fit_fx);
last_delta=delta;
last_mag=mag;
return e0;
}
#endif


void imps_do_fit(double *ret_mag,double *ret_delta,struct istruct *in,double fx,char * prefix)
{
#ifndef windows
double size=0.0;
int status=0;
int fitvars=2;
double shift=1.0;
fit_file_prefix=prefix;
inter_copy(&fit_data,in,TRUE);
double avg=inter_avg(&fit_data);
inter_sub_double(&fit_data,avg);

struct istruct peaks;
inter_init(&peaks);
inter_find_peaks(&peaks,&fit_data,TRUE);
double mag=peaks.data[0];
double time_quater=(1.0/imps_config.imps_modulation_fx)/4.0;
//printf("rod=%le %le\n",peaks.x[0],time_quater);
double delta=peaks.x[0]-time_quater;
double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};
par[0]=fx;
gsl_vector *x;
gsl_vector *ss;

x = gsl_vector_alloc (fitvars);
ss = gsl_vector_alloc (fitvars);


int pos=0;
gsl_vector_set (x, pos, mag);
gsl_vector_set (ss, pos, fabs(mag*shift));
pos++;
gsl_vector_set (x, pos, delta);
gsl_vector_set (ss, pos, fabs(delta*shift));
pos++;

	const gsl_multimin_fminimizer_type *T =	gsl_multimin_fminimizer_nmsimplex;
	gsl_multimin_function minex_func;
	gsl_multimin_fminimizer *s = NULL;

	minex_func.n = fitvars;
	minex_func.f = imps_f;
	minex_func.params = par;

	s = gsl_multimin_fminimizer_alloc (T, fitvars);
	gsl_multimin_fminimizer_set (s, &minex_func, x, ss);
	int stop=FALSE;
	int ittr=0;
	do
	{
		status = gsl_multimin_fminimizer_iterate(s);

		size = gsl_multimin_fminimizer_size (s);
		status = gsl_multimin_test_size (size, 1e-8);

		if (status == GSL_SUCCESS)
		{
			printf ("converged to minimum at\n");
		}

		if (status != GSL_CONTINUE) stop=TRUE;
		mag=gsl_vector_get(s->x, 0);
		delta=gsl_vector_get(s->x, 1);
		//printf("%s %le %10.3e delta=%10.3e\n",prefix,s->fval,mag,delta);
		if (ittr>100)
		{
			stop=TRUE;
		}
		ittr++;
	}while (stop==FALSE);

*ret_mag=last_mag;
*ret_delta=last_delta;

gsl_multimin_fminimizer_free (s);
gsl_vector_free(x);
gsl_vector_free(ss);
inter_free(&peaks);
inter_free(&fit_data);
#endif
}

double update_alpha(struct device *in,struct light *mylight,double n,double *avg)
{
double change=0.0;
int i;
int lam;
double alpha_avg=0.0;
double update=0.0;
int points=0;
double alpha0=1e7;

	for (lam=0;lam<mylight->lpoints;lam++)
	{
		for (i=0;i<mylight->points;i++)
		{
			if ((mylight->x[i]>mylight->device_start)&&(mylight->x[i]<(mylight->device_start+mylight->device_ylen)))
			{
				update=(mylight->alpha0[lam][i]-imps_config.imps_saturate*n)-mylight->alpha[lam][i];
				update=update/(1.0+fabs(update/1e5));

				mylight->alpha[lam][i]+=update;
				//if (lam==mylight->points/2) printf("%le %le %le %le\n",mylight->alpha[lam][i],mylight->alpha0[lam][i],imps_config.imps_saturate,n);
				change=change+fabs(update);
				if (mylight->alpha[lam][i]<1e4) mylight->alpha[lam][i]=1e4;
				alpha_avg+=mylight->alpha[lam][i];
				points++;
			}
		}
	}
*avg=alpha_avg/((double)points);
return change/((double)(points));
}

void sim_imps(struct device *in)
{
double i0=0.0;
char outpath[1000];
struct buffer buf;
buffer_init(&buf);
imps_load_config(&imps_config,in);
double lambda=(1.0/imps_config.imps_modulation_fx);
in->dt=lambda/imps_config.imps_points;
double stop_time=lambda*imps_config.imps_n;
int record_start=(imps_config.imps_n-1.0);




in->go_time=FALSE;

in->externalv=imps_config.imps_Vexternal;
printf("Running imps. %e\n",in->externalv);
int ittr=0;

int step=0;
light_solve_and_update(in,&(in->mylight),in->Psun*imps_config.imps_mul_sun,0.0,0.0);

sim_externalv(in,imps_config.imps_Vexternal);
remesh_reset(in,in->Vapplied);

in->Vapplied=imps_config.imps_Vexternal;
double V=in->Vapplied;

light_solve_and_update(in,&(in->mylight),1.0*imps_config.imps_mul_sun,0.0,0.0);

i0=newton_imps(in,V,FALSE);
device_timestep(in);

i0=newton_imps(in,V,FALSE);
device_timestep(in);

set_orig_charge_den(in);
double n=(get_free_n_charge_delta(in)+get_free_p_charge_delta(in))/2.0;

light_solve_and_update(in,&(in->mylight),in->Psun*imps_config.imps_mul_sun,0.0,0.0);

i0=newton_imps(in,V,FALSE);
device_timestep(in);

i0=newton_imps(in,V,FALSE);
device_timestep(in);


in->time=0.0;

time_init(in);

device_timestep(in);

in->go_time=TRUE;

struct dynamic_store store;


inter_init(&out_i);

struct istruct out_v;
inter_init(&out_v);

struct istruct out_mue;
inter_init(&out_mue);

struct istruct out_muh;
inter_init(&out_muh);

struct istruct out_mue_c;
inter_init(&out_mue_c);

struct istruct out_muh_c;
inter_init(&out_muh_c);

struct istruct out_G;
inter_init(&out_G);

struct istruct out_psun;
inter_init(&out_psun);

struct istruct out_cut_psun;
inter_init(&out_cut_psun);

struct istruct out_cut_i;
inter_init(&out_cut_i);

struct istruct n_wavelength;
inter_init(&n_wavelength);

struct istruct out_alpha_avg;
inter_init(&out_alpha_avg);

double cut_time=0.0;

reset_np_save(in);
double Plight=0.0;
int local_count=0;
int period=0;
in->time=0.0;
double alpha_max_reduction=0;
double alpha_max_increase=0;

double dc_nfree_to_ptrap=get_avg_recom_n(in);
double dc_pfree_to_ntrap=get_avg_recom_p(in);
double dc_nrelax=get_avg_relax_n(in);
double dc_prelax=get_avg_relax_p(in);

double dc_nfree=get_free_n_charge(in);
double dc_pfree=get_free_p_charge(in);
double dc_ntrap=get_n_trapped_charge(in);
double dc_ptrap=get_p_trapped_charge(in);






//double Psun_max=0.0;
dump_dynamic_init(&store,in);

do
{

	Plight=in->Psun*imps_config.imps_mul_sun+imps_config.imps_modulation_max*sin(2*PI*imps_config.imps_modulation_fx*in->time);
	//if (Plight>Psun_max)
	//{
	//	Psun_max=Plight;
	//}
	//Plight=Psun_max;

	dump_dynamic_add_data(&store,in,in->time);
	int loop=0;
	double i_last=0.0;
	double delta=0.0;
	double alpha_change=0.0;
	double avg=0.0;
	do
	{
		n=(get_free_n_charge_delta(in)+get_free_p_charge_delta(in))/2.0;
		alpha_change=update_alpha(in,&(in->mylight),n,&avg);
		light_solve_and_update(in,&(in->mylight),Plight,0.0,0.0);
		i0=newton_imps(in,V,TRUE);
		//printf("%ld %le\n",loop,alpha_change);
		loop++;
	}while(((alpha_change>5e2)&&(loop<15))||(loop<2));
	//printf("\n");
	//getchar();

	if (imps_config.imps_saturate*n<alpha_max_reduction)
	{
		alpha_max_reduction=imps_config.imps_saturate*n;
	}

	if (imps_config.imps_saturate*n>alpha_max_increase)
	{
		alpha_max_increase=imps_config.imps_saturate*n;
	}

	if (get_dump_status(dump_print_converge)==TRUE)
	{
		printf("imps time=%e step=%d error=%.1e dalpha=%.1e ",in->time,step,in->last_error,alpha_change);
		printf("Vtot=%lf current = %e mA \n",in->Vapplied,get_I(in)/1e-3);
	}

	ittr++;

	dump_write_to_disk(in);

	plot_now(in,"imps.plot");

	inter_append(&out_i,in->time,i0);
	inter_append(&out_alpha_avg,in->time,avg);
	inter_append(&out_v,in->time,V);
	inter_append(&out_mue,in->time,get_avg_mue(in));
	inter_append(&out_muh,in->time,get_avg_muh(in));
	inter_append(&out_mue_c,in->time,in->mun[0]);
	inter_append(&out_muh_c,in->time,in->mup[0]);
	inter_append(&out_G,in->time,in->Gn[0]);
	inter_append(&out_psun,in->time,Plight);
	inter_append(&n_wavelength,in->time,(double)period);

	if (period>=record_start)
	{
		inter_append(&out_cut_i,cut_time,i0);
		inter_append(&out_cut_psun,cut_time,Plight);
		cut_time+=in->dt;
	}

	device_timestep(in);
	step++;
	local_count++;

	if (local_count>=imps_config.imps_points)
	{
		period++;
		local_count=0;
	}

	if (in->time>(stop_time)) break;



	if (in->stop==TRUE)
	{
		break;

	}



}while(1);

sprintf(outpath,"%s%s",in->outputpath,"imps_alpha_avg.dat");
inter_save(&out_alpha_avg,outpath);

buffer_malloc(&buf);
buf.norm_x_axis=FALSE;
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current - Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$A$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_i.x, out_i.data, out_i.len);
buffer_dump(in->outputpath,"imps_i.dat",&buf);
buffer_free(&buf);

buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Generation rate - Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Rate");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$m^{-3}s^{-1}$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_G.x, out_G.data, out_G.len);
buffer_dump(in->outputpath,"imps_g.dat",&buf);
buffer_free(&buf);

double real=0.0;
double imag=0.0;
struct istruct out_imps;

inter_copy(&out_imps,&out_i,TRUE);
double avg=inter_avg(&out_imps);
inter_sub_double(&out_imps,avg);
inter_mul(&out_imps,-1.0);
inter_mul(&out_cut_i,-1.0);

double dx=0.0;
int i;

double max_i;
double delta_i;

double max_g;
double delta_g;



inter_save(&n_wavelength,"./n_wavelength.dat");

if (imps_config.imps_do_fit==TRUE)
{
	double nfree_to_ptrap=0.0;
	double pfree_to_ntrap=0.0;
	double nrelax=0.0;
	double prelax=0.0;

	double nfree=0.0;
	double pfree=0.0;
	double ntrap=0.0;
	double ptrap=0.0;

	double srh_n_r1=0.0;
	double srh_n_r2=0.0;
	double srh_n_r3=0.0;
	double srh_n_r4=0.0;

	if (get_dump_status(dump_dynamic)==TRUE)
	{

		struct istruct temp;
		inter_copy(&temp,&(store.jout),TRUE);
		inter_sub_double(&temp,temp.data[0]);
		inter_mul(&temp,-1.0);
		int xpos=inter_get_max_pos(&temp);
		inter_free(&temp);
		nfree=store.nfree.data[xpos]-dc_nfree;
		pfree=store.pfree.data[xpos]-dc_pfree;
		ntrap=store.ntrap.data[xpos]-dc_ntrap;
		ptrap=store.ptrap.data[xpos]-dc_ptrap;


		nfree_to_ptrap=store.nfree_to_ptrap.data[xpos]-dc_nfree_to_ptrap;
		pfree_to_ntrap=store.pfree_to_ntrap.data[xpos]-dc_pfree_to_ntrap;
		nrelax=store.nrelax_out.data[xpos]-dc_nrelax;
		prelax=store.prelax_out.data[xpos]-dc_prelax;

		srh_n_r1=store.srh_n_r1.data[xpos];
		srh_n_r2=store.srh_n_r2.data[xpos];
		srh_n_r3=store.srh_n_r3.data[xpos];
		srh_n_r4=store.srh_n_r4.data[xpos];

	}
	imps_do_fit(&max_g,&delta_g,&out_cut_psun,imps_config.imps_modulation_fx,"g");
	imps_do_fit(&max_i,&delta_i,&out_cut_i,imps_config.imps_modulation_fx,"i");

	printf("delta_i=%lf delta_g=%lf\n",delta_i,delta_g);
	double dphi=2*PI*imps_config.imps_modulation_fx*(delta_i-delta_g);
	printf("delta_phi=%lf\n",dphi);

	double mag=max_i;
	real=mag*cos(dphi);
	imag=mag*sin(dphi);


	//inter_dft(&real,&imag,&out_i,imps_config.imps_modulation_fx);
	double ang=atan2(imag,real)*360/(2.0*PI);
	printf("real=%le imag=%le ang=%lf\n",real,imag,ang);

	double band_bend=inter_get_max_range(&(store.band_bend),store.band_bend.len/2,store.band_bend.len);

	FILE *out;
	out=fopena(in->outputpath,"./sim_info.dat","w");
	fprintf(out,"#imps_r\n%le\n",real);
	fprintf(out,"#imps_i\n%le\n",imag);
	fprintf(out,"#imps_Jr\n%le\n",real/in->area);
	fprintf(out,"#imps_Ji\n%le\n",imag/in->area);
	fprintf(out,"#imps_fx\n%le\n",imps_config.imps_modulation_fx);
	fprintf(out,"#imps_delta_i\n%le\n",delta_i);
	fprintf(out,"#imps_delta_g\n%le\n",delta_g);
	fprintf(out,"#imps_delta_phase\n%le\n",dphi);
	fprintf(out,"#max_nfree_to_ptrap\n%le\n",nfree_to_ptrap);
	fprintf(out,"#max_pfree_to_ntrap\n%le\n",pfree_to_ntrap);
	fprintf(out,"#max_nrelax\n%le\n",nrelax);
	fprintf(out,"#max_prelax\n%le\n",prelax);
	fprintf(out,"#max_nfree\n%le\n",nfree);
	fprintf(out,"#max_pfree\n%le\n",pfree);
	fprintf(out,"#max_ntrap\n%le\n",ntrap);
	fprintf(out,"#max_ptrap\n%le\n",ptrap);
	fprintf(out,"#alpha_max_reduction\n%le\n",alpha_max_reduction);
	fprintf(out,"#alpha_max_increase\n%le\n",alpha_max_increase);

	fprintf(out,"#srh_n_r1\n%le\n",srh_n_r1);
	fprintf(out,"#srh_n_r2\n%le\n",srh_n_r2);
	fprintf(out,"#srh_n_r3\n%le\n",srh_n_r3);
	fprintf(out,"#srh_n_r4\n%le\n",srh_n_r4);
	fprintf(out,"#band_bend_max\n%le\n",band_bend);
	fclose(out);
}

buffer_malloc(&buf);

struct istruct out_j;
inter_copy(&out_j,&out_cut_i,TRUE);
double avg_out_j=inter_avg(&out_j);
inter_sub_double(&out_j,avg_out_j);
inter_div_double(&out_j,in->area);

buf.norm_x_axis=FALSE;
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current density- Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current denisty");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$A/m^2$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_j.x, out_j.data, out_j.len);
buffer_dump(in->outputpath,"imps_j.dat",&buf);
buffer_free(&buf);


inter_rescale(&out_i,1.0/out_i.x[out_i.len-1], 1.0);
buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current - Time (norm)");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$Amps$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_i.x, out_i.data, out_i.len);
buffer_dump(in->outputpath,"imps_i_norm_x.dat",&buf);
buffer_free(&buf);

inter_rescale(&out_i,1.0/out_i.x[out_i.len-1], 1.0);
avg=inter_avg(&out_i);
inter_sub_double(&out_i,avg);
inter_mul(&out_i,-1.0);

buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Current - Time (norm)");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Current");
strcpy(buf.x_units,"$au$");
strcpy(buf.y_units,"$Amps$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_i.x, out_i.data, out_i.len);
buffer_dump(in->outputpath,"imps_i_norm_x_ax_only.dat",&buf);
buffer_free(&buf);

buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Voltage - Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Voltage");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$Volts$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_v.x, out_v.data, out_v.len);
buffer_dump(in->outputpath,"imps_v.dat",&buf);
buffer_free(&buf);

sprintf(outpath,"%s%s",in->outputpath,"imps_mue.dat");
inter_save(&out_mue,outpath);

sprintf(outpath,"%s%s",in->outputpath,"imps_muh.dat");
inter_save(&out_muh,outpath);

sprintf(outpath,"%s%s",in->outputpath,"imps_mue_c.dat");
inter_save(&out_mue_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"imps_muh_c.dat");
inter_save(&out_muh_c,outpath);

avg=inter_avg(&out_psun);
inter_sub_double(&out_psun,avg);



buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1.0;
strcpy(buf.title,"Suns - Time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Suns");
strcpy(buf.x_units,"$s$");
strcpy(buf.y_units,"$Sun$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_psun.x, out_psun.data, out_psun.len);
buffer_dump(in->outputpath,"imps_psun.dat",&buf);
buffer_free(&buf);


in->go_time=FALSE;

inter_free(&out_j);
inter_free(&out_G);
inter_free(&out_muh);
inter_free(&out_mue);
inter_free(&out_muh_c);
inter_free(&out_mue_c);
inter_free(&out_i);
inter_free(&out_v);
inter_free(&out_psun);
inter_free(&out_imps);
inter_free(&out_alpha_avg);

inter_free(&out_cut_i);
inter_free(&out_cut_psun);
inter_free(&n_wavelength);

dump_dynamic_save(in->outputpath,&store);
dump_dynamic_free(&store);
}

void imps_load_config(struct imps *in,struct device *dev)
{
struct inp_file inp;
inp_init(&inp);
inp_load_from_path(&inp,dev->inputpath,"imps.inp");
inp_check(&inp,1.27);
inp_search_double(&inp,&(in->imps_points),"#imps_points");
inp_search_double(&inp,&(in->imps_n),"#imps_n");
inp_search_double(&inp,&(in->imps_Vexternal),"#imps_Vexternal");
inp_search_double(&inp,&(in->Rshort_imps),"#Rshort_imps");
inp_search_double(&inp,&(in->imps_modulation_max),"#imps_modulation_max");
inp_search_double(&inp,&(in->imps_saturate),"#imps_saturate");

in->imps_modulation_max=fabs(in->imps_modulation_max);

if (in->imps_modulation_max>0.9)
{
	in->imps_modulation_max=0.9;
}
inp_search_double(&inp,&(in->imps_modulation_fx),"#imps_modulation_fx");

inp_search_double(&inp,&(dev->Rext),"#Rext");
inp_search_double(&inp,&(dev->Cext),"#Cext");

inp_search_double(&inp,&(in->imps_mul_sun),"#imps_mul_sun");
in->imps_mul_sun=fabs(in->imps_mul_sun);

if (in->imps_mul_sun<0.01)
{
	in->imps_mul_sun=0.01;
}

inp_search_int(&inp,&(in->imps_do_fit),"#imps_do_fit");

inp_free(&inp);



}
